-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

newTalent{
	name = "Primal Combat",
	type = {"wild-gift/wyrmic-combat", 1},
	mode = "passive",
	require = gifts_req1,
	points = 5,
	on_learn = function(self, t)
		self:attr("allow_any_dual_weapons", 1)
	end,
	on_unlearn = function(self, t)
		if not self:knowTalent(t) then
			self:attr("allow_any_dual_weapons", -1)
		end
	end,
	getCriticalChance = function(self, t) return 4 + self:getTalentLevel(t) * self:getWil() / 20 end,
	info = function(self, t)
		local critchance = t.getCriticalChance(self, t)
		return ([[Allows you to dual wield any type of one handed weapons and increases the chances of your dual weapons attacks to be critical hits by %d%%. After scoring a critical hit you will recover %d equilibrium]]):format(critchance, self:getTalentLevel(t)/2)
	end,
}

newTalent{
	name = "Elemental Charge",
	type = {"wild-gift/wyrmic-combat", 2},
	mode = "sustained",
	require = gifts_req2,
	points = 5,
	sustain_equilibrium = 10,
	cooldown = 20,
	tactical = { BUFF = 2 },
	getDamage = function(self, t) return self:combatTalentMindDamage(t, 4, 20) end,
	on_pre_use = function(self, t, silent)
		local weapon, offweapon = self:hasDualWeapon()
		if not  weapon then
			if not silent then 
				game.logPlayer(self, "You require two weapons to use this talent.")
			end
			return false 
		end 
		return true
	end,
	activate = function(self, t)
		local weapon, offweapon = self:hasDualWeapon()
		if not weapon then
			game.logPlayer(self, "You cannot use Elemental Charge without two weapons!")
			return nil
		end
		return {
			atk = self:addTemporaryValue("combat_atk", 4 + self:getTalentLevel(t) * self:getWil() / 20)
		}
	end,
	deactivate = function(self, t, p)
		self:removeTemporaryValue("combat_atk", p.atk)
		return true
	end,
	info = function(self, t)
		local damage = t.getDamage(self, t)
		local atk = 4 + self:getTalentLevel(t) * self:getWil() / 20
		return ([[Imbues both your weapons with the destructive power of the wyrms. While active, it adds %0.2f fire, %0.2f cold and %0.2f lightning damage to your hits and raise your accuracy by %d. At level 4 it will start to hit near foes and increase your equilibrium by 2 at each hit. The damage will increase with the Willpower stat.]]):
		format(damDesc(self, DamageType.FIRE, damage), damDesc(self, DamageType.COLD, damage), damDesc(self, DamageType.LIGHTNING, damage), atk)
	end,
}

newTalent{
	name = "Double Claws",
	type = {"wild-gift/wyrmic-combat", 3},
	require = gifts_req3,
	points = 5,
	equilibrium = 6,
	cooldown = 8,
	range = 1,
	radius = 1,
	requires_target = true,
	getDuration = function(self, t)
		return 2 + self:getTalentLevel(t) / 3
	end,
	tactical = {ATTACK = {FIRE = 1, COLD = 1}, DISABLE = 1},
	action = function(self, t)
		local weapon, offweapon = self:hasDualWeapon()
		if not weapon then
			game.logPlayer(self, "You cannot use Double Claws without two weapons!")
			return nil
		end
		
		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y or not target then return nil end
		if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end

		self.combat_physcrit = self.combat_physcrit + 10 + self.combat_physcrit + self:getTalentLevel(t) * 5
		local speed1, hit1 = self:attackTargetWith(target, weapon.combat, DamageType.FIRE, self:combatTalentWeaponDamage(t, 0.6, 1.4))

		if hit1 then
			
			local speed2, hit2 = self:attackTargetWith(target, offweapon.combat, DamageType.COLD, self:getOffHandMult(self:combatTalentWeaponDamage(t, 0.6, 1.4)))
			if hit2 and target:canBe("confusion") then
				target:setEffect(target.EFF_CONFUSED, 2, {apply_power=self:combatPhysicalpower()})
			else
				game.logSeen(self, "%s resists the confusion.", target.name:capitalize())
			end
		end
		self.combat_physcrit = self.combat_physcrit - 10 - self.combat_physcrit + self:getTalentLevel(t) * 5

		return true
	end,
	info = function(self, t)
		local dur = t.getDuration(self,t)
		return ([[Hits the target with your main weapon, doing %d%% fire weapon damage. If the attack hits, you attack with your second weapon, doing %d%% cold weapon damage. If this attack hits the target is confused for %d turns. This powerful attack has an aditional %d%% chance of becoming critical.]]):
		format(100 * self:combatTalentWeaponDamage(t, 0.6, 1.4), 100 * self:combatTalentWeaponDamage(t, 0.6, 1.4), dur, 10 + self:getTalentLevel(t) * 5)
	end,
}

newTalent{
	name = "Grand Smasher",
	type = {"wild-gift/wyrmic-combat", 4},
	require = gifts_req4,
	points = 5,
	equilibrium = 8,
	cooldown = 10,
	tactical = { ATTACK = { weapon = 1 }, EQUILIBRIUM = 0.5},
	getDuration = function(self, t)
		return 2 + self:getTalentLevel(t) / 2
	end,
	requires_target = true,
	action = function(self, t)
		local weapon, offweapon = self:hasDualWeapon()
		if not weapon then
			game.logPlayer(self, "You cannot use Grand Smasher without two weapons!")
			return nil
		end
		
		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y or not target then return nil end
		if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end

		local  hit1 = self:attackTarget(target, nil, self:combatTalentWeaponDamage(t, 0.8, 1.3), true)
		if hit1 and target:canBe("stun") then
			target:setEffect(target.EFF_STUNNED, t.getDuration(self, t), {apply_power=self:combatPhysicalpower()})
		else
			game.logSeen(self, "%s resists the stunning smash.", target.name:capitalize())
		end
		
		local  hit2 = self:attackTarget(target, nil, self:combatTalentWeaponDamage(t, 0.9, 1.4), true)
		if hit2 and target:canBe("pin") then
			target:setEffect(target.EFF_PINNED, t.getDuration(self, t), {apply_power=self:combatPhysicalpower()})
		else
			game.logSeen(self, "%s resists the pinning smash.", target.name:capitalize())
		end
		
		return true
	end,
		
	info = function(self, t)
		local dur = t.getDuration(self, t)
		return ([[Smashes the target twice with both weapons. The first attack does %d%% weapon damage and tries to stun the target for %d turns. The second attack does %d%% weapon damage and tries to pin the target for %d turns.]]):
		format(100 * self:combatTalentWeaponDamage(t, 0.8, 1.3), dur, 100 * self:combatTalentWeaponDamage(t, 1, 1.5), dur)
	end,
}